# Script Locked by: twarlan
# Script Version: 2.0.2.4
# Script Name: app_debitcard
# Script Lock Date: Wed Apr 17 14:08:56 2002
#         
# Copyright (c) 2001 by Cisco Systems, Inc.
# All rights reserved.
#------------------------------------------------------------------
#         
# Description: 
#         
# The debitcard application allows a user to to select the lanaguage 
# mode based on the langauges that are configured through the CLIs.
# The application then prompts and collects the card number. The
# card number consists of a uid and pin where uid-length and pin-length
# are configured through through CLIs. 
# Authentication is done with the card number. If it passes authentication,
# the application plays the amount available in the user's debitcard. It then
# prompts and collects the destination number. 
# If authentication fails, the application allows user to retry and the number
# of retries is configured through the CLI.
# Authorization is done with the destination number. If authorization is
# successful, the application plays a time duration which indicates the amount of 
# talk time in the user's debitcard account and places the call. 
# If authorization fails, the application allows users to retry.
#         
# The main routine is at the bottom.
#         
# Modifications (5/12):
#         
# - Added new playprompt usage
# - Notify user if AAA server is dead
# - Spanish lang support
# - For account with zero credit, only allow them to make toll-free calls
# - type ahead with Long Pound feature
#         
# Modifications (6/7)
#         
# - Play failure tone instead of a prompt when place_call fails
# - Play a prompt when an invalid language is selected
#
# Modifications (6/24)
# - Enable long pound detection only for debitcard application
#
# Modifications (6/25)
# - Deleted while loop at the end of the script
# - Script will loop back to start at beginning.
#
# Modifications (7/7)
# - new verb setVariable
# - Usage : setVariable language 1
#           setvariable longpound true
# - replaced all language prefixes (en_, sp_) with genric prefixes (prefix1, prefix2 etc)
#
# Modifications (7/13)
# - Modified prompt playing sequence in select_language for primary and secondary 
#   languages
#
# Modifications (8/6)
# - Allow 1 language support
# - After long pound, skip the first authorization request, ie. creditamt is not 
#   played back to the user
# - Play error prompts depending on return codes for Radius
#
# Modification CSCdu08506
# - Fix for not playing message when caller is out of time
# - Script should say it is disconnecting.
# - Need to split connection destroy and media play
#
# Modification (4/30/2001)
# - Fix for newguid generate
# - new conf-id after longPound
#
# Modification (5/10/2001)
# - Nirmala Shukla 
# - Modified script for longpound (Illayaraja) featurette.
#
# Modification (10/22/2001)
# - CSCdv54414 long pound calls have same GUID
# - CSCdv48808 long pound during ring does not work
#
# Modification (1/2/2002)
# - CSCdw24067 fix to generate a new h323-conf-id for calls
#              after called party hangup
# Modification (2/2/2002)
# - CSCdw24067 fix to generate a new h323-conf-id for calls
#              after called party hangup
# Modification (2/14/2002)
# - CSCdv54436 Fix the problem application aborts when # or non
#              numeric key is entered during language selection
# 
# Modification (4/4/2002)
# - CSCdx20442 Long pound functionality does not function fully.
#   Remove comment for playtone in act_TerminateSetup
#
#----------------------------------------------------------------

proc do_hthousands {} {
    global prompt
    global numbers

    set hlen [string length $numbers(hthousands)]
    lappend prompt _$numbers(hthousands)00.au
    puts "\t\t*** Hthousands Prompt: $prompt"
    return 0
}

proc do_thousands {} {
    global prompt
    global numbers
    global len

    if {$numbers(thousands) == 0} {
            lappend prompt _thousand.au
    } else {
            	 if {$len == 4} {
                      switch -regexp $numbers(thousands) {
                          1     {lappend prompt _thousand.au}
                          2     {lappend prompt _$numbers(thousands)_f.au _2-4_thousand.au}
                        {[3-4]} {lappend prompt _$numbers(thousands).au _2-4_thousand.au}
                        default {lappend prompt _$numbers(thousands).au _5-20_thousand.au}
                      }
                 } else {
                      set seperate [split $numbers(thousands) ""]
                      switch -regexp $numbers(thousands) {
                          {00}       {lappend prompt _thousand.au}
                          {01}       {lappend prompt _thousand.au}
                          {02}       {lappend prompt lappend prompt _[lindex $seperate 1]_f.au _2-4_thousand.au}
                          {[0][3-4]} {lappend prompt lappend prompt _[lindex $seperate 1].au _2-4_thousand.au}
                          {[0][5-9]} {lappend prompt lappend prompt _[lindex $seperate 1].au _5-20_thousand.au}
                          {[1-9][0]} {lappend prompt _[lindex $seperate 0]0.au _5-20_thousand.au}
                          {[1][1-9]} {lappend prompt _$numbers(thousands).au _5-20_thousand.au}
                          {[2-9][1]} {lappend prompt _$numbers(thousands)_f.au _thousand.au}
                          {[2-9][2]} {lappend prompt _$numbers(thousands)_f.au _2-4_thousand.au}
                          {[2-9][3-4]} {lappend prompt _[lindex $seperate 0]0.au _[lindex $seperate 1].au _2-4_thousand.au}
                          {[2-9][5-9]} {lappend prompt _[lindex $seperate 0]0.au _[lindex $seperate 1].au _5-20_thousand.au}
                      }
                 }
    }
    puts "\n*** Thousands Prompt: $prompt"
    return 0
}

proc do_hundreds {} {
    global prompt
    global numbers

    if {$numbers(hundreds) == 0} {
       return 0
    } else {
       lappend prompt _$numbers(hundreds)00.au
    }
    puts "\n*** Hundreds Prompt: $prompt"
    return 0
}

proc do_tens {} {
    global prompt
    global numbers
    global len
    global gender

    if {$numbers(tens) != 0} {
            	  if {[string length $numbers(tens)] == 1} {
                       switch -regexp $numbers(tens) {
                           {0} { }
                           {1-2} {lappend prompt _$numbers(tens)_[set gender].au}
                           default {lappend prompt _$numbers(tens).au}
                       }
                   } else { 
			set seperate [split $numbers(tens) ""]
                        switch -regexp $numbers(tens) {
                            {00}         { }
                            {[0][1-2]}   {lappend prompt _[lindex $seperate 1]_[set gender].au}
                            {[0][3-9]}   {lappend prompt _[lindex $seperate 1].au}
                            {[1][1-9]}   {lappend prompt _$numbers(tens).au}
			    {[2-9][1-2]} {lappend prompt _[lindex $seperate 0]0.au _[lindex $seperate 1]_[set gender].au}
                            {[1-9][0]}   {lappend prompt _[lindex $seperate 0]0.au}
                            default      {lappend prompt _[lindex $seperate 0]0.au _[lindex $seperate 1].au}
                        }
		   }
    }
    puts "\n*** $numbers(tens) Tens Prompt: $prompt"
    return 0
}

proc do_whole_part {} {
    global amt
    global prompt
    global numbers
    global len
    global newlist

    puts "do_whole_part: $newlist"

    set numbers(tens) ""
    set numbers(hundreds) ""
    set numbers(thousands) ""
    set numbers(hthousands) ""

    set len [string length [lindex $newlist 0]]

    set seperate [split [lindex $newlist 0] ""]

    puts "\t\t*** TTS whole: $seperate"

# Do the number group translation

    switch -regexp $len {
        {1} {set numbers(tens) [lindex $seperate 0]
        }
        {2} {set numbers(tens) [join [list [lindex $seperate 0] [lindex $seperate 1]] "" ]
        }
        {3} {set numbers(hundreds) [lindex $seperate 0]
             set numbers(tens) [join [list [lindex $seperate 1] [lindex $seperate 2]] "" ]
        }
        {4} {set numbers(thousands) [lindex $seperate 0]
             set numbers(hundreds) [lindex $seperate 1]
             set numbers(tens) [join [list [lindex $seperate 2] [lindex $seperate 3]] "" ]
        }
        {5}  {set numbers(thousands) [join [list [lindex $seperate 0] [lindex $seperate 1]] "" ]
             set numbers(hundreds) [lindex $seperate 2]
             set numbers(tens) [join [list [lindex $seperate 3] [lindex $seperate 4]] "" ]
        }
        {6}  {set numbers(hthousands) [lindex $seperate 0]
             set numbers(thousands) [join [list [lindex $seperate 1] [lindex $seperate 2]] "" ]
             set numbers(hundreds) [lindex $seperate 3]
             set numbers(tens) [join [list [lindex $seperate 4] [lindex $seperate 5]] "" ]
        }
    }

    foreach index {hthousands thousands hundreds tens} {

       if {[string compare $numbers($index) ""] != 0} {
	   puts "\t\t*** $index $numbers($index)"
           do_$index
       }
   }
   return 0
}

proc do_decimal_part {} {
    global amt
    global prompt
    global numbers
    global len
    global newlist

    puts "do_decimal_part: $newlist"

    set numbers(tens) ""
    set numbers(hundreds) ""
    set numbers(thousands) ""
    set numbers(hthousands) ""

    set len [string length [lindex $newlist 1]]

    set seperate [split [lindex $newlist 1] ""]

    puts "\t\t*** TTS decimal: $seperate"

# Do the number group translation

    switch -regexp $len {
        {1} {set numbers(tens) [lindex $seperate 0]
        }
        {2} {set numbers(tens) [join [list [lindex $seperate 0] [lindex $seperate 1]] "" ]
        }
        {3} {set numbers(hundreds) [lindex $seperate 0]
             set numbers(tens) [join [list [lindex $seperate 1] [lindex $seperate 2]] "" ]
        }
        {4} {set numbers(thousands) [lindex $seperate 0]
             set numbers(hundreds) [lindex $seperate 1]
             set numbers(tens) [join [list [lindex $seperate 2] [lindex $seperate 3]] "" ]
        }
        {5}  {set numbers(thousands) [join [list [lindex $seperate 0] [lindex $seperate 1]] "" ]
             set numbers(hundreds) [lindex $seperate 2]
             set numbers(tens) [join [list [lindex $seperate 3] [lindex $seperate 4]] "" ]
        }
        {6}  {set numbers(hthousands) [lindex $seperate 0]
             set numbers(thousands) [join [list [lindex $seperate 1] [lindex $seperate 2]] "" ]
             set numbers(hundreds) [lindex $seperate 3]
             set numbers(tens) [join [list [lindex $seperate 4] [lindex $seperate 5]] "" ]
        }
    }

    foreach index {hthousands thousands hundreds tens} {

       if {[string compare $numbers($index) ""] != 0} {
	   puts "\t\t*** $index $numbers($index)"
           do_$index
       }
   }
   return 0
}

proc init { } {
    global param1
    global beep
    global param2
    global param3
    global pound
    global prompt

    set beep 0
    set pound 0
    set outgoingDisconnect 0

    set param1(interruptPrompt) true
    set param1(abortKey) #
    set param1(terminationKey) *

    
    set param2(interruptPrompt) true
    set param2(abortKey) *
    set param2(terminationKey) #
    set param2(dialPlanTerm) true

    set param3(interruptPrompt) true
    set param3(abortKey) *
    set param3(terminationKey) #
}

proc init_ConfigVars { } {
    global accountLen
    global pinLen
    global retryCnt
    global operatorNum
    global warnTime
    global totalLang

    puts "\n-----------------------------------------> Init_ConfigVar <-----------"

    set accountLen [string trim [infotag get cfg_avpair uid-len]]
    set pinLen [string trim [infotag get cfg_avpair  pin-len]]
    set retryCnt [string trim [infotag get  cfg_avpair retry-count]]
    set operatorNum [string trim [infotag get cfg_avpair  redirect-number]]
#    set warnTime [string trim [infotag get cfg_avpair  warning-time]]
    set warnTime 60

    #Language related variables
    set totalLang [string trim [infotag get med_total_languages]]
}

proc init_perCallVars { } {
    global selectCnt
    global count 
    global fcnt
    global ani
    global dontPlayAmount
    global destBusy
    global noTimeLimit
    global promptFlag
    global promptFlag2
    global noPlay 
    global outgoingDisconnect
    global commandHandle
    global playBalance

    puts "\n-----------------------------------------> Init_CallVar <-----------"
    set selectCnt 0
    set count 0
    set fcnt 0
    set dontPlayAmount 0
    set destBusy 0
    set noTimeLimit 0
    set promptFlag 0
    set promptFlag2 0
    set noPlay 0
    set outgoingDisconnect 0
    set commandHandle ""

    set ani [infotag get leg_ani]
    set playBalance 0
} 

proc act_Setup { } {
    global totalLang
    global param1
    
    puts "\n-----------------------------------------> Setup <-----------"
    init_perCallVars
    infotag set med_language 1

    leg setupack leg_incoming
    leg proceeding leg_incoming
    leg connect leg_incoming

    #Lang selection
    puts "\n-----------------------------------------> TotalLanguages=$totalLang <-----------"
    if { $totalLang < 1 } {
	act_Cleanup
	return
    }
    
    if { $totalLang == 1 } {
#	media play leg_incoming _welcome.au
#	fsm setstate ONELANG
        fsm setstate CHECKANI
	act_CheckANI
        return 
    }

    set param1(maxDigits) 1
    leg collectdigits leg_incoming param1 

    #Get the map and use it...not this way ???
    media play leg_incoming _welcome.au %s1000 %c1 _lang_sel1.au %s1000 %c2 _lang_sel2.au 
    
}

proc act_LangSelect { } {
    global param1
    global selectCnt
    global retryCnt
   
    set status [infotag get evt_status]
    #Collect Success ...matched pattern
    if {$status == "cd_005"} {
          set input [infotag get evt_dcdigits]

	if {![regexp {^([0-9])$} $input d langId]} { 
                 incr selectCnt
         	 if {$selectCnt < $retryCnt} {
	         leg collectdigits leg_incoming param1
	         media play leg_incoming  %c1 _no_lang_sel.au %s1000 %c1 _lang_sel1.au %s1000 %c2 _lang_sel2.au
	         fsm setstate same_state
                } else {
	         media play leg_incoming %c1 _generic_final.au
	         fsm setstate CALLDISCONNECT
	       }
                return
	} 
	set status [infotag set med_language [infotag get evt_dcdigits] ]
        
	if { ($status == "language not supported") || 
	     ($status == "language not specified in configuration") } {
	    
	    incr selectCnt
	    if {$selectCnt < $retryCnt} {
		#Get the map and use it...not this way ???
		
		set param1(maxDigits) 1
		leg collectdigits leg_incoming param1
                
		media play leg_incoming  %c1 _wrong_lang_sel.au %s1000 %c1 _lang_sel1.au %s1000 %c2 _lang_sel2.au
	
		fsm setstate same_state
		return 
            } else {
		media play leg_incoming %c1 _generic_final.au
		fsm setstate CALLDISCONNECT
		return 
            }
	}
	#Get the card No
	act_GetCard
    }
    
    #Collect timeout/Abort
    if { ($status == "cd_001") ||
	 ($status == "cd_002")} {

	incr selectCnt
	if {$selectCnt < $retryCnt} {
	    leg collectdigits leg_incoming param1
	    media play leg_incoming  %c1 _no_lang_sel.au %s1000 %c1 _lang_sel1.au %s1000 %c2 _lang_sel2.au
	   
	    fsm setstate same_state
        } else {
	    media play leg_incoming %c1 _generic_final.au
	    fsm setstate CALLDISCONNECT
        }
    }	     
     	
}

proc act_GetCard { } {

    global cardLen
    global accountLen
    global pinLen
    global count
    global param1
    global promptFlag

    puts "\n-------------------------> act_GetCard <-----------------------------"

    set cardLen [expr $accountLen + $pinLen]

#    set param1(maxDigits) $cardLen 
    set param1(maxDigits) [expr $accountLen + 1]

    leg collectdigits leg_incoming param1 
    if {$count == 0} {
#	media play leg_incoming _enter_card_num.au
        media play leg_incoming %s250 _enter_card_num.au
    } else {
	if {$promptFlag == 1 } {
#	    media play leg_incoming  _invalid_digits.au
#            media play leg_incoming  _incorrect_card_num.au %s250 _enter_card_num.au
            media play leg_incoming  _dest_blocked.au
	} elseif {$promptFlag == 2 } {
#	    media play leg_incoming _auth_fail.au
            media play leg_incoming  _incorrect_card_num.au %s250 _enter_card_num.au
	} elseif {$promptFlag == 3} {
	    media play leg_incoming _no_card_entered.au
	} elseif {$promptFlag == 10} {
	    media play leg_incoming _generic_final.au %s250 _enter_card_num.au
#rebel	    media play leg_incoming _card_in_use.au %s250 _enter_card_num.au
	} elseif {$promptFlag == 11} {
	    media play leg_incoming _zero_bal.au %s250 _enter_card_num.au
	}
    }

}

proc act_GetPin { } {

    global pinLen
    global param1
    global promptFlag
    global account
    global ani

    puts "\n-------------------------> act_GetPin <------------------------------"

    set account [infotag get evt_dcdigits]

    set param1(maxDigits) [expr $pinLen + 1]

    leg collectdigits leg_incoming param1 
}

proc act_CheckANI { } {
    global ani
    global account
    global pin

    puts "\n-------------------------> act_Checkani -$ani- <---------"

    set account $ani
    set pin "ThisIsANI"

    if {$account == ""} {
	fsm setstate GETCARD
        act_GetCard
    } else {
        aaa authorize $account $pin $ani "" leg_incoming
        return 
    }

}

proc act_GotCardNumber { } {
    global count
    global retryCnt
    global promptFlag
    global account
    global pin
    global accountLen
    global cardLen
    global ani

    set status [infotag get evt_status]

    puts "\n-------------------------> act_GotCardNumber Status=$status <---------"

    # Do we need to calculate len ???
    if {$status == "cd_005"} {
	    set pin [infotag get evt_dcdigits]
	    #Do authorize
	    aaa authorize $account $pin  $ani "" leg_incoming
	return 
    }
    
    #timeout
    if {$status == "cd_001"} {
	set number [infotag get evt_dcdigits]
	puts "\nNUMBER=$number"
	set numberLen [ string length $number ]
	if { $numberLen == 0 } {
	    incr count
	    set promptFlag 3
	    if {$count < $retryCnt} {
		act_GetCard
		fsm setstate same_state
	    } else {
		media play leg_incoming _oss_thankyou.au
		fsm setstate CALLDISCONNECT
		
	    }
	    return
	}
	if { $numberLen == $cardLen } {
            set account [ string range $number 0 [expr $accountLen -1]]
            set pin [ string range $number $accountLen [expr $cardLen -1]]
	    
	    #Do authorize
	    aaa authorize $account $pin  $ani "" leg_incoming
        } else {
            incr count
            set promptFlag 1
            if {$count < $retryCnt} {
		act_GetCard
		fsm setstate same_state
            } else {
		media play leg_incoming _oss_thankyou.au
		fsm setstate CALLDISCONNECT
	    }
        }
	return 
    }
    
    #Aborted
    if {$status == "cd_002"} {
	act_GetCard
	fsm setstate same_state
	return
    }
    
    #timeout/fail(disconnected)/invalid_no
    #shd we play a diff prompt for disconnected...what is disconnected???
    if {($status == "cd_001") || 
	($status == "cd_007") || 
	($status == "cd_006")} {
	incr count
        set promptFlag 3
        if {$count < $retryCnt} {
	    act_GetCard
	    fsm setstate same_state
        } else {
	    media play leg_incoming _oss_thankyou.au
	    fsm setstate CALLDISCONNECT
	}
	return
    }
}

proc	act_GotANI { } {
    set status [infotag get evt_status]
    global amt

    puts "\n-------------------------------> aaa authorize ANI Status=$status <-----------"
    if { $status == "ao_000" } {
        if { [infotag get aaa_avpair_exists h323-credit-amount] } {
            set amt [infotag get aaa_avpair h323-credit-amount]
        } else {
            media play leg_incoming _no_aaa.au
            fsm setstate CALLDISCONNECT
            return 
        }

        act_GetDest
        return 
    } else {
        if { [infotag get aaa_avpair_exists h323-return-code] } {
            set returnCode [infotag get aaa_avpair h323-return-code]
        } else {
            #No return code from radius
            media play leg_incoming _no_aaa.au
            fsm setstate CALLDISCONNECT
            return 
        }

	fsm setstate GETCARD
        act_GetCard
	return
    }

}

proc act_FirstAuthorized { } {
    global count
    global retryCnt
    global promptFlag
    global amt

    set status [infotag get evt_status]
    
    puts "\n-------------------------------> aaa authorize Status=$status <-----------"
    if { $status == "ao_000" } {
        if { [infotag get aaa_avpair_exists h323-credit-amount] } {
	    set amt [infotag get aaa_avpair h323-credit-amount]
	} else {
	    media play leg_incoming _no_aaa.au
	    fsm setstate CALLDISCONNECT
	    return 
	}

	act_GetDest
	return 
    } else {
	if { [infotag get aaa_avpair_exists h323-return-code] } {
	    set returnCode [infotag get aaa_avpair h323-return-code]
	    puts "\n-------------------------------> aaa RC=$returnCode <-----------"
	} else {
	    #No return code from radius
            media play leg_incoming _no_aaa.au
	    fsm setstate CALLDISCONNECT
            return 
	}

	incr count
	if {$returnCode == 3} {
	        set promptFlag 10
	} elseif {$returnCode == 4} {
	        set promptFlag 11
	} else {
	        set promptFlag 2
	}
        if { $count < $retryCnt } {
            fsm setstate GETCARD
	    act_GetCard
#	    fsm setstate CARDSELECTION
        } else {
	    #First authorize failed
            media play leg_incoming _oss_thankyou.au
	    fsm setstate CALLDISCONNECT
        }
        return 0
	
    }
    fsm setstate CALLDISCONNECT
}

proc act_TerminateSetup { } {
    global duration
    global commandHandle

    puts "\n\t\t**** act_TerminateSetup"

    if { [infotag get evt_digit] != "#" } { 
	fsm setstate same_state
    } else {
        set duration [infotag get evt_digit_duration]

        if { $duration < 1000 } {
	    fsm setstate same_state
	} else {
            # Longpound detected

            set returnCode [command terminate $commandHandle]
            puts "\n supress any tones that might be active \n"
            playtone leg_incoming "tn_none"

            if { $returnCode == 0 } {

	       # continue to wait for the ev_setup_done

	    } elseif { $returnCode == 1 } {

	       # The call setup command is done, no event will be returned

	       act_Just_GetDest
	       fsm setstate GETDEST

	   } elseif { $returnCode == 2 } {
	       # Invalid command.  Maybe setup had not been initiated, attempt to
               # wait for it.
   	       puts "\n\t\t**** Oops, command terminate command invalid!"
	       fsm setstate PLACECALL
	   }
	}
    }
}

proc act_GetDest { } {
    global fcnt
    global amt
    global promptFlag2
    global destBusy
    global param2
    global pound
    global playBalance
    global gender
    global numbers
    global newlist
    global prompt
 
    set param2(enableReporting) true
    set param2(ignoreInitialTermKey) false
    set param2(maxDigits) 25

    puts "\n-------------------------------> GetDest $playBalance $amt <-------"

    leg collectdigits leg_incoming param2

    if {$fcnt == 0} {
#        if {$amt == 0} {
#           media play leg_incoming _zero_bal.au 
#	    fsm setstate CALLDISCONNECT
#       } else
	if {$amt > 999999.99} {
            media play leg_incoming _invalid_amt.au _dollars.au %s250 _enter_dest.au
        } elseif {$destBusy == 1} {
	    set destBusy 0
            media play leg_incoming _dest_busy.au
	} elseif {$pound == 1} {
	    set pound 0
            media play leg_incoming _disconnect.au _enter_dest.au
        } else {
#	    media play leg_incoming _you_have.au %a$amt %s1000 _enter_dest.au
	if {$playBalance == 0} {
		 media play leg_incoming _enter_dest.au
	     } else {
		    set fDigit [string range $amt 0 0]
		    if {$fDigit == "-"} {
			media play leg_incoming _zero_bal.au %s250 _enter_dest.au
		    } else {
			    set prompt ""

	               	    puts "russian amt started $amt"
        	            set newlist [split $amt .]
       	        	    set gender m
	                    do_whole_part

			    puts "----------------------------> $prompt"

                	    set tens_len [string length $numbers(tens)]
	       	            puts "TENS LEN: $tens_len"
        	       	    if {$tens_len == 1 } {
                	        switch -regexp $numbers(tens) {
       	                	    {1} {lappend prompt _dollar.au}
	               	            {[2-4]} {lappend prompt _2-4_dollars.au}
        	             	    {[5-9]} {lappend prompt _5-20_dollars.au}
                	        }
	       	            } else {
        	       	        switch -regexp $numbers(tens) {
                	     	    {[1][0-9]} {lappend prompt _5-20_dollar.au}
                	     	    {[2-9][1]} {lappend prompt _dollar.au}
                        	    {[2-9][2-4]} {lappend prompt _2-4_dollars.au}
	       	                    {[2-9][5-9]} {lappend prompt _5-20_dollars.au}
	       	                    {[2-9][0]} {lappend prompt _5-20_dollars.au}
        	       	            {[0][0]} {lappend prompt _5-20_dollars.au}
                	     	    {[0][1]} {lappend prompt _dollar.au}
                        	    {[0][2-4]} {lappend prompt _2-4_dollars.au}
	       	                    {[0][5-9]} {lappend prompt _5-20_dollars.au}
                	        }
	       	            }
       		

			    puts "----------------------------> $prompt"

        	            set gender m
       	        	    do_decimal_part

	                    if {[string length [lindex $newlist 1]] == 1} {
       		                  switch -regexp [lindex $newlist 1] {
               		              {1}     {lappend prompt _cent.au}
                       		      {[2-4]} {lappend prompt _2-4_cents.au}
	                              {[5-9]} {lappend prompt _5-20_cents.au}
       		                  }
               		    } else {
                       		  switch -regexp [lindex $newlist 1] {
	                              {[1-9][0]} {lappend prompt _5-20_cents.au}
	                              {[1][1-9]} {lappend prompt _5-20_cents.au}
       		                      {[2-9][1]}   {lappend prompt _cent.au}
               		              {[2-9][2-4]} {lappend prompt _2-4_cents.au}
                       		      {[2-9][5-9]} {lappend prompt _5-20_cents.au}
       		                      {[0-9][1]}   {lappend prompt _cent.au}
               		              {[0-9][2-4]} {lappend prompt _2-4_cents.au}
                       		      {[0-9][5-9]} {lappend prompt _5-20_cents.au}
	                          }
        	            }

			    puts "----------------------------> $prompt"
		            media play leg_incoming _you_have.au $prompt %s250 _enter_dest.au
			    set playBalance 0
		    }
 	     }
        }
    } else {
         switch -regexp $promptFlag2 {
	     {1} { media play leg_incoming _reenter_dest.au }
	     {2} { media play leg_incoming _dest_blocked.au %s250 _enter_dest.au}
	     {3} { media play leg_incoming _no_dest_entered.au }
	 }
    } 
}

proc act_GotDest { } {
    global fcnt
    global retryCnt
    global promptFlag2
    global destination
    global ani
    global account
    global pin
    global dontPlayAmount
    global playBalance

    set status [infotag get evt_status]

    set status [infotag get evt_status]
    puts "\n\n-----------------------------> act_GotDest Status=$status <------------" 
    if {$status == "cd_005"} {
        set destination [infotag get evt_dcdigits]
	if {$destination == ""} {
	  set playBalance 1
          fsm setstate GETDEST
	  act_GetDest
	} else {

	  set dpref [string match "810*" "$destination"]
	  if {$dpref == 1} {
	      set destination [string range $destination 3 end]
	  } else {
	    set dpref [string match "8*" "$destination"]
	    if {$dpref == 1} {
		 set destination [string range $destination 1 end]
		 set destination "7$destination"
	    } else {
		    set destination "7495$destination"
	    }
	  }

#	  set destination "002$destination"

          aaa authorize $account $pin  $ani $destination leg_incoming
	}
        return
    }

    #matched dialplan
    if {$status == "cd_004"} {
	set destination [infotag get evt_dcdigits]
	aaa authorize $account $pin  $ani $destination leg_incoming
	return
    }

    #aborted
    if {$status == "cd_002"} {
	if {$dontPlayAmount == 1} {
	    act_Just_GetDest
	} else {
	    act_GetDest
	}
	fsm setstate same_state
	return 
    }


    incr fcnt
    if {$fcnt < $retryCnt} {
	#timeout
	#shd we play a diff prompt for disconnected...what is disconnected???
	if {$status == "cd_001" } {
	    set promptFlag2 1
	}
	#fail(disconnected)
	if {$status == "cd_007" } {
	    set promptFlag2 3
	}	
	#for all other cases like cd_006(invalid no)
	set promptFlag2 1
	if {$dontPlayAmount == 1} {
	    act_Just_GetDest
	} else {
	    act_GetDest
	}
	fsm setstate same_state
    } else {
	media play leg_incoming _dest_collect_fail.au
	fsm setstate CALLDISCONNECT
    }
    
    
}

proc act_SecondAuthorized { } {
    global account
    global destination
    global noTimeLimit
    global creditTime
    global warnTime
    global promptFlag2
    global fcnt
    global retryCnt
    global noPlay
    global commandHandle
    
    set callInfo(accountNum) $account

    puts "\n-----------------------------> act_SecondAuthorized Account=$account <-----" 

    if { [infotag get aaa_avpair_exists h323-return-code] } {
	set returnCode [infotag get aaa_avpair h323-return-code]
    } else {
	#No return code from radius
	media play leg_incoming _no_aaa.au
	fsm setstate CALLDISCONNECT
	return 
    }
    
    puts "\n-----------------------------> act_SecondAuthorized ReturnCode=$returnCode <-----" 
     if {$returnCode == 13} {
	 # allow user to make toll-free calls
	 set noTimeLimit 1 
	 leg setup $destination callInfo leg_incoming
         set commandHandle [infotag get last_command_handle]
	 return 
     }
     #this is same as above ???
     if {$returnCode == 52} {
	 # Redirect to customer service using redirect number
	 set noTimeLimit 1 
	 leg setup $destination callInfo leg_incoming
         set commandHandle [infotag get last_command_handle]
	 return 
     }

    set status [infotag get evt_status]
    puts "\n --------------------------------> aaa second authorize Status=$status"

    if { $status == "ao_000" } {
	if { [infotag get aaa_avpair_exists h323-credit-time] } {
	    set creditTime [infotag get aaa_avpair h323-credit-time]
	} else {
	    media play leg_incoming _no_aaa.au
	    fsm setstate CALLDISCONNECT
	    return 
	}

        puts "\n --------------------------------> aaa second authorize creditTime=$creditTime"
	if {$creditTime == "uninitialized"} {
            media play leg_incoming _no_aaa.au
	    fsm setstate CALLDISCONNECT
	    return
        } elseif {$creditTime == "unlimited"} {
	    set noTimeLimit 1
	    # play mesg only if time left is  20 secs > warntime
        } elseif {[expr $creditTime - $warnTime] < 20} {
	    set noPlay 1
            media play leg_incoming _oss_null.au
#	    media play leg_incoming _you_have.au %t$creditTime 
        } else {
#	    media play leg_incoming _you_have.au %t$creditTime 
	    if {$creditTime < 3600} {
#              media play leg_incoming _you_have.au %t$creditTime
              media play leg_incoming _oss_null.au
	    } else {
              media play leg_incoming _oss_null.au
	    }
        }

    } else {
	#Authorization failed
	act_PlayReturnCode $returnCode
#	if { $returnCode == 4 } {
#	    #special Case ???...no retry check also
#            set creditTime [infotag get aaa_avpair h323-credit-time]
#            media play leg_incoming _oss_null.au
#	    return 
#	} else {
#	    fsm setstate CALLDISCONNECT
#	    return 
#	}

	incr fcnt
        set promptFlag2 2
	if { $fcnt < $retryCnt } {
	    puts "\n --------------------------------> SA Retry $fcnt on $retryCnt ----"
	    fsm setstate GETDEST
            act_GetDest 
	    
        } else {
            #second authorize failed

            media play leg_incoming _oss_thankyou.au
	    fsm setstate CALLDISCONNECT
        }
#	fsm setstate CALLDISCONNECT
    }
}

proc act_PlayReturnCode {returnCode} {

    switch $returnCode {
	3  { media play leg_incoming _account_blocked.au }
	4  { media play leg_incoming  _zero_bal.au }
	5  { media play leg_incoming _card_expired.au }
        7  { media play leg_incoming _blocked.au }
        9  { media play leg_incoming _dest_blocked.au }
        12 { media play leg_incoming _not_enuf.au }
	default { media play leg_incoming _no_aaa.au }
    }
    
}

proc act_CallSetup { } {
    global destination
    global account
    global pound
    global commandHandle
    global outgoingDisconnect

    puts "\n\n-----------------------------> act_CallSetup" 

    if { ($pound == 1) || ($outgoingDisconnect == 1) } {
         set callInfo(newguid) true
    }

    set callInfo(accountNum) $account
    leg setup $destination callInfo leg_incoming
    set commandHandle [infotag get last_command_handle]
}

proc act_CallSetupDone { } {
    global warnTime
    global creditTime
    global noPlay
    global noTimeLimit
    global param3
    global pound
    global outgoingDisconnect

    set status [infotag get evt_status]

    set param3(interruptPrompt) true
    set param3(enableReporting) true
    
    puts "\n-----------------------------> act_CallSetupDone Status = $status" 

    if {$status == "ls_000"} {
        set pound 0
        set outgoingDisconnect 0
	
	if { ($noTimeLimit == 1) || 
	     ($creditTime == "unlimited") ||
	     ($creditTime == "uninitialized") } {
	    #no timer
	} else {
	    if {$creditTime < $warnTime} {
		if {$noPlay == 1} {
		    timer start leg_timer  [expr $creditTime - 1] leg_incoming
		} else {
		    timer start leg_timer [expr $warnTime - 4] leg_incoming
		}
		fsm setstate CALLLASTACTIVE

	    } else {
		set delay [expr $creditTime - $warnTime]
		timer start leg_timer $delay leg_incoming
		fsm setstate CALLACTIVE
	    }
	}
        return

    #call setup timeout/invalid no/Destination Busy/dest disconnect/
    #conference failed/

    } elseif { ($status == "ls_002") || 
             ($status == "ls_003") ||
	     ($status == "ls_004") ||
	     ($status == "ls_007") ||
	     ($status == "ls_009") ||
	     ($status == "ls_010") 
	   } {
	      act_DestBusy
	      return
    } elseif { $status == "ls_005" 
           } {	

	    set param2(ignoreInitialTermKey) false

	    leg collectdigits leg_incoming param2
            media play leg_incoming _notcall.au %s500 _enter_dest.au
	    fsm setstate GETDEST
	    return
    } else {
        # leg setupFail

        set tone [infotag get evt_status]
        playtone leg_incoming $tone
        timer start leg_timer 5 leg_incoming 
        fsm setstate CALLDISCONNECT
    }
}

proc act_HandleOutgoing { } {
    global outgoingDisconnect

    if { [infotag get evt_legs] == [infotag get leg_outgoing] } {
	puts "\nOutgoing Disconnected"
	connection destroy con_all
	set outgoingDisconnect 1
	return
    }
    call close 
    fsm setstate CALLDISCONNECT

}


proc act_LongPound { } {
    global duration
    global pound

    if { [infotag get evt_digit] != "#" } { 
	fsm setstate same_state
	return 
    }

    set duration [infotag get evt_digit_duration]
    if { $duration < 1000 } {
        fsm setstate same_state
	return 
    } else {
        set pound 1
        connection destroy con_all
    } 
}

proc act_Just_GetDest { } {
    global fcnt
    global promptFlag2
    global pound
    global destBusy
    global param2
    global outgoingDisconnect 

    puts "\n\n-----------------------------> act_Just_GetDest" 

    set param2(enableReporting) true
    set param2(ignoreInitialTermKey) false

    leg collectdigits leg_incoming param2
    if {$fcnt == 0} {
	if {$outgoingDisconnect == 1} {
	    set outgoingDisconnect 0
	    media play leg_incoming _disconnect.au %s250 _enter_dest.au
	} elseif {$destBusy == 1} {
	    set destBusy 0
	    media play leg_incoming _dest_busy.au
	} elseif {$pound == 1} {
	      set pound 0
              media play leg_incoming _disconnect.au _enter_dest.au
	} else {
	    media play leg_incoming _enter_dest.au
	}
    } else {
        if {$promptFlag2 == 1} {
            media play leg_incoming _reenter_dest.au
        }
        if {$promptFlag2 == 2} {
            media play leg_incoming _author_fail.au
        }
        if {$promptFlag2 == 3} {
            media play leg_incoming _no_dest_entered.au
        }
    } 
}

proc act_ConnDestroyed { } {
    global dontPlayAmount 

    leg disconnect leg_outgoing
    set dontPlayAmount 1
    act_Just_GetDest
}

proc act_DestBusy { } {
    global destBusy
    
    #current debitcard implementation
    set destBusy 1
    act_Just_GetDest
    fsm setstate GETDEST
}

proc act_ActiveTimer { } {
    global noPlay
    global warnTime
    global creditTime
    global incoming
    global outgoing

    set incoming [infotag get leg_incoming]
    set outgoing [infotag get leg_outgoing]

    if {$noPlay == 1} {
	timer start leg_timer  [expr $creditTime -1] leg_incoming
    } else {
        connection destroy con_all
	timer start leg_timer [expr $warnTime - 4] leg_incoming
        fsm setstate CALLWARN
    }
}

proc act_CallWarnDestroy { } {
#    global warnTime
    media play leg_incoming _you_have.au _1_a.au _minute.au
}

proc act_CallWarnMedia { } {
    global incoming
    global outgoing

    connection create $incoming $outgoing  
}

proc act_ConnectionDestroy { } {
    connection destroy con_all
}

proc act_LastActiveTimer { } {
#    media play leg_incoming _disconnect.au
    media play leg_incoming _oss_thankyou.au
}

proc act_Ignore { } {
# Dummy
    puts "Event Capture"
}

proc act_Cleanup { } {
    puts "\n\n\t\t-----------------------------> act_Cleanup <--------------"
    call close
}

requiredversion 2.0
init 

#Get Variables
init_ConfigVars

#----------------------------------
#   State Machine
#----------------------------------
  set fsm(any_state,ev_disconnected)           "act_Cleanup           same_state"
    
  set fsm(CALL_INIT,ev_setup_indication)       "act_Setup             LANGSELECTION"

  set fsm(ONELANG,ev_media_done)               "act_CheckANI          CHECKANI"
#  set fsm(ONELANG,ev_media_done)               "act_GetCard           CARDSELECTION"

  set fsm(CHECKANI,ev_authorize_done)          "act_GotANI            GETDEST"

  set fsm(GETCARD,ev_collectdigits_done)       "act_GetPin            CARDSELECTION"

  set fsm(LANGSELECTION,ev_collectdigits_done) "act_LangSelect        CARDSELECTION"

  set fsm(CARDSELECTION,ev_collectdigits_done) "act_GotCardNumber     AUTHORIZE"
  
  set fsm(AUTHORIZE,ev_authorize_done)         "act_FirstAuthorized   GETDEST"

  set fsm(GETDEST,ev_collectdigits_done)       "act_GotDest           SECONDAUTHORIZE"

  set fsm(SECONDAUTHORIZE,ev_authorize_done)   "act_SecondAuthorized  PLACECALL"

  set fsm(PLACECALL,ev_media_done)             "act_CallSetup         PLACECALL"
  set fsm(PLACECALL,ev_setup_done)             "act_CallSetupDone     CALLACTIVE"
  set fsm(PLACECALL,ev_digit_end)              "act_TerminateSetup    WAITFORSETUPDONE"

  set fsm(WAITFORSETUPDONE,ev_setup_done)       "act_Just_GetDest     GETDEST"

  set fsm(CALLACTIVE,ev_leg_timer)             "act_ActiveTimer       CALLLASTACTIVE"
  set fsm(CALLACTIVE,ev_digit_end)             "act_LongPound         CONNDESTROY"
  set fsm(CALLACTIVE,ev_disconnected)          "act_HandleOutgoing    CONNDESTROY"

  set fsm(CALLLASTACTIVE,ev_leg_timer)         "act_ConnectionDestroy PLAYDISCONNECT"
  set fsm(CALLLASTACTIVE,ev_digit_end)         "act_LongPound         CONNDESTROY"

  set fsm(PLAYDISCONNECT,ev_destroy_done)      "act_LastActiveTimer   CALLDISCONNECT"

  set fsm(CALLWARN,ev_destroy_done)            "act_CallWarnDestroy   same_state"
  set fsm(CALLWARN,ev_media_done)              "act_CallWarnMedia     CALLLASTACTIVE"

  set fsm(CONNDESTROY,ev_destroy_done)         "act_ConnDestroyed     GETDEST"

  set fsm(CALLDISCONNECT,ev_media_done)        "act_Cleanup           same_state"
  set fsm(CALLDISCONNECT,ev_disconnect_done)   "act_Cleanup           same_state"
# set fsm(CALLDISCONNECT,ev_leg_timer)         "act_Cleanup           same_state"
  set fsm(CALLDISCONNECT,ev_leg_timer)         "act_ConnDestroyed     GETDEST"
  
  fsm define fsm CALL_INIT

# Script Approval Signature: C/b445
